@@ -37,6 +37,7 @@ type cappingAction string
 var (
 	cappedToMinAllowed             cappingAction = "capped to minAllowed"
 	cappedToMaxAllowed             cappingAction = "capped to maxAllowed"
+	cappedToLimit                  cappingAction = "capped to container limit"
 	cappedProportionallyToMaxLimit cappingAction = "capped to fit Max in container LimitRange"
 	cappedProportionallyToMinLimit cappingAction = "capped to fit Min in container LimitRange"
 )
@@ -108,6 +109,7 @@ func getCappedRecommendationForContainer(
 	}
 	// containerPolicy can be nil (user does not have to configure it).
 	containerPolicy := GetContainerResourcePolicy(container.Name, policy)
+	containerControlledValues := GetContainerControlledValues(container.Name, policy)
 
 	cappedRecommendations := containerRecommendation.DeepCopy()
 
@@ -121,6 +123,13 @@ func getCappedRecommendationForContainer(
 			cappingAnnotations = append(cappingAnnotations, limitAnnotations...)
 			cappingAnnotations = append(cappingAnnotations, annotations...)
 		}
+		// TODO: If limits and policy are conflicting, set some condition on the VPA.
+		if containerControlledValues == vpa_types.ContainerControlledValuesRequestsOnly {
+			annotations = capRecommendationToContainerLimit(recommendation, container)
+			if genAnnotations {
+				cappingAnnotations = append(cappingAnnotations, annotations...)
+			}
+		}
 	}
 
 	process(cappedRecommendations.Target, true)
@@ -130,6 +139,21 @@ func getCappedRecommendationForContainer(
 	return cappedRecommendations, cappingAnnotations, nil
 }
 
+// capRecommendationToContainerLimit makes sure recommendation is not above current limit for the container.
+// If this function makes adjustments appropriate annotations are returned.
+func capRecommendationToContainerLimit(recommendation apiv1.ResourceList, container apiv1.Container) []string {
+	annotations := make([]string, 0)
+	// Iterate over limits set in the container. Unset means Infinite limit.
+	for resourceName, limit := range container.Resources.Limits {
+		recommendedValue, found := recommendation[resourceName]
+		if found && recommendedValue.MilliValue() > limit.MilliValue() {
+			recommendation[resourceName] = limit
+			annotations = append(annotations, toCappingAnnotation(resourceName, cappedToLimit))
+		}
+	}
+	return annotations
+}
+
 // applyVPAPolicy updates recommendation if recommended resources are outside of limits defined in VPA resources policy
 func applyVPAPolicy(recommendation apiv1.ResourceList, policy *vpa_types.ContainerResourcePolicy) []string {
 	if policy == nil {
